"""Implementation of XBAspecies class.

Peter Schubert, HHU Duesseldorf, May 2022
"""

import sbmlxdf
from xbanalysis.utils.utils import xml_gba_ns, xml_rba_ns


class XbaSpecies:

    def __init__(self, s_species):
        self.id = s_species.name
        self.name = s_species.get('name', self.id)
        # set sboterm to 000247 to support Flux Balance Analysis
        self.sboterm = s_species.get('sboterm', 'SBO:0000247')
        self.compartment = s_species['compartment']
        self.constant = s_species['constant']
        self.boundary = s_species['boundaryCondition']
        if 'initialConcentration' in s_species:
            self.initial_conc = s_species['initialConcentration']
        if 'units' in s_species:
            self.units = s_species['substanceUnits']
        if 'xmlAnnotation' in s_species:
            attrs = sbmlxdf.extract_xml_attrs(s_species['xmlAnnotation'], ns=xml_gba_ns, token='molecule')
            if 'weight_Da' in attrs:
                self.mw = float(attrs['weight_Da'])
            attrs = sbmlxdf.extract_xml_attrs(s_species['xmlAnnotation'], ns=xml_rba_ns, token='costs')
            if len(attrs) > 0:
                if ('process_machine' in attrs) and ('process_capacity_per_s' in attrs):
                    self.rba_process_machine = attrs.pop('process_machine')
                    self.rba_process_capacity = float(attrs.pop('process_capacity_per_s'))
                self.rba_process_costs = attrs

        self.ps_rid = None
        self.m_reactions = {}
        self.ps_reactions = {}
        self.p_degrad_rids = {}
        self.p_trans_rids = {}

    def set_ps_rid(self, rid):
        self.ps_rid = rid

    def add_m_reaction(self, rid, stoic):
        self.m_reactions[rid] = stoic

    def add_ps_reaction(self, rid, stoic):
        self.ps_reactions[rid] = stoic

    def add_p_degrad_rid(self, rid, stoic):
        self.p_degrad_rids[rid] = stoic

    def add_p_transition_rid(self, rid, stoic):
        self.p_trans_rids[rid] = stoic
